#ifndef TDS3DBufferH
#define TDS3DBufferH

// ==========================================================================
// File:         TDS3DBuffer.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               
// Description:  This file defines the TDS3DBuffer Component 
//
// "TDx_Sound_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Sound_Library_Defns.H"
#include "TDx_Sound_Library_Functions.H"

// ==========================================================================
// Set:         TDS3DBufferMode_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TDS3DBuffer::Mode property using the BCB Object Inspector. 
//              For detailed descriptions of each flag, see the Mode property 
//              description below. 
// ==========================================================================
enum TDS3DBuffer_Mode_Flags
    {
    ds3dmode_disable,
    ds3dmode_headrelative,
    ds3dmode_normal
    };
typedef Set< TDS3DBuffer_Mode_Flags, 
             ds3dmode_disable, 
             ds3dmode_normal >  TDS3DBuffer_Mode_Set;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDS3DBuffer
// Description: The TDS3DBuffer component wraps the DirectX DS3DBUFFER 
//              structure which describes the location, sound cones, 
//              orientation and motion of a 3D sound buffer. 
//              
//              This component is used when retrieving or setting all the 
//              properties of a 3D sound buffer at once using 
//              TDx_Sound3DBuffer::GetAllParameters() and 
//              TDx_Sound3DBuffer::SetAllParameters(). 
//              
//              For 3D sound to be effective at least one 3D sound buffer and 
//              one 3D sound listener is required. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDS3DBuffer : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDS3DBuffer : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Mode_
    // Description: The Mode property defines flags indicating the 3D sound 
    //              processing mode that is used for the 3D sound buffer. 
    //              The described effect applies when the flag is set. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'Mode' property via the BCB Object Inspector. 
    //               For detailed descriptions of each flag, see the Mode 
    //              property description below. 
    // ----------------------------------------------------------------------
    __property TDS3DBuffer_Mode_Set Mode_ = { read=FGetMode_, 
    write=FSetMode_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    ConeOutsideVolume
    // Description: The ConeOutsideVolume property defines, in 100ths of a 
    //              decibel, the cone outside volume. 
    // ----------------------------------------------------------------------
    __property long ConeOutsideVolume = { read=FGetConeOutsideVolume, 
    write=FSetConeOutsideVolume, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    InsideConeAngle
    // Description: The InsideConeAngle property defines the angle, in 
    //              degrees, of the inside sound projection cone. 
    // ----------------------------------------------------------------------
    __property dword InsideConeAngle = { read=FGetInsideConeAngle, 
    write=FSetInsideConeAngle, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    OutsideConeAngle
    // Description: The OutsideConeAngle property defines the angle, in 
    //              degrees, of the outside sound projection cone. 
    //              This angle must always equal or exceed that of the inside 
    //              sound projection cone angle. 
    // ----------------------------------------------------------------------
    __property dword OutsideConeAngle = { read=FGetOutsideConeAngle, 
    write=FSetOutsideConeAngle, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property defines the size, in bytes, of the 
    //              internal structure used by this component. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, 
    default=sizeof(DS3DBUFFER) }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDS3DBuffer component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ConeOrientation
    // Description: The ConeOrientation property defines a D3DVECTOR which 
    //              describes the current orientation of the 3D sound cone. 
    //              This vector represents the center of the sound cone. 
    // ----------------------------------------------------------------------
    __property D3DVECTOR ConeOrientation = { read=FGetConeOrientation, 
    write=FSetConeOrientation, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDS3DBuffer method or 
    //              fget/fset. eg. DS_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DS_OK }; 

    // ----------------------------------------------------------------------
    // Property:    MaxDistance
    // Description: The MaxDistance property defines a D3DVALUE indicating, 
    //              usually in metres, a sounds maximum distance. 
    //              A sound may be turned off when maximum distance is 
    //              reached by setting DSBCAPS_MUTE3DATMAXDISTANCE when 
    //              creating the 3D sound buffer. 
    // ----------------------------------------------------------------------
    __property D3DVALUE MaxDistance = { read=FGetMaxDistance, 
    write=FSetMaxDistance, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    MinDistance
    // Description: The MinDistance property defines a D3DVALUE indicating, 
    //              usually in metres, a sounds minimum distance. 
    //              The default value for this property is 1. 
    // ----------------------------------------------------------------------
    __property D3DVALUE MinDistance = { read=FGetMinDistance, 
    write=FSetMinDistance, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Mode
    // Description: The Mode property defines flags indicating the 3D sound 
    //              processing mode that is used for the 3D sound buffer. 
    //              The described effect applies when the flag is set. 
    // Flags:       DS3DMODE_DISABLE -
    //                Disable all 3D sound processing. 
    //                With no 3D processing, sounds will seem to originate 
    //                from the center of the listeners head. 
    //              DS3DMODE_HEADRELATIVE -
    //                Make all properties, like a sounds position, velocity 
    //                and orientation, relative to those of the listener. 
    //                The absolute properties of the sound are modified 
    //                automatically whenever the listeners parameters change. 
    //                This results in the sounds relative parameters 
    //                remaining constant. 
    //              DS3DMODE_NORMAL -
    //                Position and orient the sound source absolutely in 
    //                world space. 
    //                This is the default processing mode. 
    // ----------------------------------------------------------------------
    __property dword Mode = { read=FGetMode, write=FSetMode, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Position
    // Description: The Position property defines a D3DVECTOR which describes 
    //              the current position of the 3D sound buffer. 
    // ----------------------------------------------------------------------
    __property D3DVECTOR Position = { read=FGetPosition, write=FSetPosition, 
    nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Velocity
    // Description: The Velocity property defines a D3DVECTOR which describes 
    //              the current velocity of the 3D sound buffer. 
    // ----------------------------------------------------------------------
    __property D3DVECTOR Velocity = { read=FGetVelocity, write=FSetVelocity, 
    nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDS3DBuffer's internal DS3DBUFFER structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Property:    Mode_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the Mode property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* Mode_Strings = { read=FGetMode_Strings };

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDS3DBuffer(TComponent* Owner); 
    virtual __fastcall ~TDS3DBuffer(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DS3DBUFFER* Internal_DS3DBUFFER_Ptr = { 
    read=FGetInternal_DS3DBUFFER_Ptr, nodefault }; 

    void __fastcall Internal_DS3DBUFFER_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Object Inspector 'Flag' Access Methods
    // ----------------------------------------------------------------------
    TDS3DBuffer_Mode_Set __fastcall FGetMode_(); 
    void __fastcall FSetMode_( TDS3DBuffer_Mode_Set pMode_ ); 

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    D3DVECTOR __fastcall FGetConeOrientation(); 
    void __fastcall FSetConeOrientation( D3DVECTOR pConeOrientation ); 
    long __fastcall FGetConeOutsideVolume(); 
    void __fastcall FSetConeOutsideVolume( long pConeOutsideVolume ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetInsideConeAngle(); 
    void __fastcall FSetInsideConeAngle( dword pInsideConeAngle ); 
    D3DVALUE __fastcall FGetMaxDistance(); 
    void __fastcall FSetMaxDistance( D3DVALUE pMaxDistance ); 
    D3DVALUE __fastcall FGetMinDistance(); 
    void __fastcall FSetMinDistance( D3DVALUE pMinDistance ); 
    dword __fastcall FGetMode(); 
    void __fastcall FSetMode( dword pMode ); 
    dword __fastcall FGetOutsideConeAngle(); 
    void __fastcall FSetOutsideConeAngle( dword pOutsideConeAngle ); 
    D3DVECTOR __fastcall FGetPosition(); 
    void __fastcall FSetPosition( D3DVECTOR pPosition ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    D3DVECTOR __fastcall FGetVelocity(); 
    void __fastcall FSetVelocity( D3DVECTOR pVelocity ); 

    // ----------------------------------------------------------------------
    // Flag_Strings Access Methods
    // ----------------------------------------------------------------------
    TStringList* __fastcall FGetMode_Strings();

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DS3DBUFFER* __fastcall FGetInternal_DS3DBUFFER_Ptr(); 
    void __fastcall FSetInternal_DS3DBUFFER( DS3DBUFFER* pDS3DBUFFER ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Flags_Strings Variables
    // ----------------------------------------------------------------------
    TStringList* fMode_Strings;

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DS3DBUFFER fDS3DBUFFER; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
