#ifndef TDSCapsH
#define TDSCapsH

// ==========================================================================
// File:         TDSCaps.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               
// Description:  This file defines the TDSCaps Component 
//
// "TDx_Sound_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Sound_Library_Defns.H"
#include "TDx_Sound_Library_Functions.H"

// ==========================================================================
// Set:         TDSCapsFlags_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TDSCaps::Flags property using the BCB Object Inspector. For 
//              detailed descriptions of each flag, see the Flags property 
//              description below. 
// ==========================================================================
enum TDSCaps_Flags_Flags
    {
    dscaps_certified_,
    dscaps_continuousrate,
    dscaps_emuldriver,
    dscaps_primary16bit,
    dscaps_primary8bit,
    dscaps_primarymono,
    dscaps_primarystereo,
    dscaps_secondary16bit,
    dscaps_secondary8bit,
    dscaps_secondarymono,
    dscaps_secondarystereo
    };
typedef Set< TDSCaps_Flags_Flags, 
             dscaps_certified_, 
             dscaps_secondarystereo >  TDSCaps_Flags_Set;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDSCaps
// Description: The TDSCaps component wraps the DirectX DSCAPS structure 
//              which defines the capabilities of a sound device. 
//              
//              This component is used when retrieving the capabilities of 
//              the sound hardware with TDx_Sound::GetCaps(). 
//              
//              The properties of this component describe the maximum and 
//              currently available sound hardware resources such as hardware 
//              mixing channels, memory available for various buffer types 
//              and so on. 
//              
//              Some resources may be shared among several buffer types, so 
//              assigning a buffer of one type may impinge upon the resources 
//              available for another type. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDSCaps : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDSCaps : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Flags_
    // Description: The Flags property defines flags specifying the available 
    //              device capabilities. 
    //              The described effect applies when the flag is set. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'Flags' property via the BCB Object Inspector. 
    //               For detailed descriptions of each flag, see the Flags 
    //              property description below. 
    // ----------------------------------------------------------------------
    __property TDSCaps_Flags_Set Flags_ = { read=FGetFlags_, 
    write=FSetFlags_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    FreeHw3DAllBuffers
    // Description: The FreeHw3DAllBuffers property defines the amount of 
    //              unallocated 3D positional capability the hardware has 
    //              available for all buffers. 
    //              By comparing this property with 
    //              TDSCaps::MaxHw3DAllBuffers, a value for the resources 
    //              already allocated can be determined. 
    // ----------------------------------------------------------------------
    __property dword FreeHw3DAllBuffers = { read=FGetFreeHw3DAllBuffers, 
    write=FSetFreeHw3DAllBuffers, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    FreeHw3DStaticBuffers
    // Description: The FreeHw3DStaticBuffers property defines the amount of 
    //              unallocated 3D positional capability the hardware has 
    //              available for static buffers. 
    //              By comparing this property with 
    //              TDSCaps::MaxHw3DStaticBuffers, a value for the resources 
    //              already allocated can be determined. 
    // ----------------------------------------------------------------------
    __property dword FreeHw3DStaticBuffers = { 
    read=FGetFreeHw3DStaticBuffers, write=FSetFreeHw3DStaticBuffers, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    FreeHw3DStreamingBuffers
    // Description: The FreeHw3DStreamingBuffers property defines the amount 
    //              of unallocated 3D positional capability the hardware has 
    //              available for streaming buffers. 
    //              By comparing this property with 
    //              TDSCaps::MaxHw3DStreamingBuffers, a value for the 
    //              resources already allocated can be determined. 
    // ----------------------------------------------------------------------
    __property dword FreeHw3DStreamingBuffers = { 
    read=FGetFreeHw3DStreamingBuffers, write=FSetFreeHw3DStreamingBuffers, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    FreeHwMemBytes
    // Description: The FreeHwMemBytes property defines the amount, in bytes, 
    //              of memory on the sound card that is unallocated. 
    // ----------------------------------------------------------------------
    __property dword FreeHwMemBytes = { read=FGetFreeHwMemBytes, 
    write=FSetFreeHwMemBytes, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    FreeHwMixingAllBuffers
    // Description: The FreeHwMixingAllBuffers property defines the amount of 
    //              unallocated mixing capability the hardware has available 
    //              for all buffers. 
    //              By comparing this property with 
    //              TDSCaps::MaxHwMixingAllBuffers, a value for the resources 
    //              already allocated can be determined. 
    // ----------------------------------------------------------------------
    __property dword FreeHwMixingAllBuffers = { 
    read=FGetFreeHwMixingAllBuffers, write=FSetFreeHwMixingAllBuffers, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    FreeHwMixingStaticBuffers
    // Description: The FreeHwMixingStaticBuffers property defines the amount 
    //              of unallocated mixing capability the hardware has 
    //              available for static buffers. 
    //              By comparing this property with 
    //              TDSCaps::MaxHwMixingStaticBuffers, a value for the 
    //              resources already allocated can be determined. 
    // ----------------------------------------------------------------------
    __property dword FreeHwMixingStaticBuffers = { 
    read=FGetFreeHwMixingStaticBuffers, write=FSetFreeHwMixingStaticBuffers, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    FreeHwMixingStreamingBuffers
    // Description: The FreeHwMixingStreamingBuffers property defines the 
    //              amount of unallocated mixing capability the hardware has 
    //              available for streaming buffers. 
    //              By comparing this property with 
    //              TDSCaps::MaxHwMixingStreamingBuffers, a value for the 
    //              resources already allocated can be determined. 
    // ----------------------------------------------------------------------
    __property dword FreeHwMixingStreamingBuffers = { 
    read=FGetFreeHwMixingStreamingBuffers, 
    write=FSetFreeHwMixingStreamingBuffers, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    MaxContigFreeHwMemBytes
    // Description: The MaxContigFreeHwMemBytes property defines the size, in 
    //              bytes, of the largest contiguous block of memory on the 
    //              sound card that is unallocated. 
    // ----------------------------------------------------------------------
    __property dword MaxContigFreeHwMemBytes = { 
    read=FGetMaxContigFreeHwMemBytes, write=FSetMaxContigFreeHwMemBytes, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    MaxHw3DAllBuffers
    // Description: The MaxHw3DAllBuffers property defines the total amount 
    //              of 3D positional capability the hardware has for all 
    //              buffers. 
    //              By comparing this property with 
    //              TDSCaps::FreeHw3DAllBuffers, a value for the resources 
    //              already allocated can be determined. 
    // ----------------------------------------------------------------------
    __property dword MaxHw3DAllBuffers = { read=FGetMaxHw3DAllBuffers, 
    write=FSetMaxHw3DAllBuffers, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    MaxHw3DStaticBuffers
    // Description: The MaxHw3DStaticBuffers property defines the total 
    //              amount of 3D positional capability the hardware has for 
    //              static buffers. 
    //              By comparing this property with 
    //              TDSCaps::FreeHw3DStaticBuffers, a value for the resources 
    //              already allocated can be determined. 
    // ----------------------------------------------------------------------
    __property dword MaxHw3DStaticBuffers = { read=FGetMaxHw3DStaticBuffers, 
    write=FSetMaxHw3DStaticBuffers, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    MaxHw3DStreamingBuffers
    // Description: The MaxHw3DStreamingBuffers property defines the total 
    //              amount of 3D positional capability the hardware has for 
    //              streaming buffers. 
    //              By comparing this property with 
    //              TDSCaps::FreeHw3DStreamingBuffers, a value for the 
    //              resources already allocated can be determined. 
    // ----------------------------------------------------------------------
    __property dword MaxHw3DStreamingBuffers = { 
    read=FGetMaxHw3DStreamingBuffers, write=FSetMaxHw3DStreamingBuffers, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    MaxHwMixingAllBuffers
    // Description: The MaxHwMixingAllBuffers property defines the total 
    //              number of mixing buffers the hardware offers for all 
    //              buffer types. 
    //              By comparing this property with 
    //              TDSCaps::FreeHwMixingAllBuffers, a value for the 
    //              resources already allocated can be determined. 
    //              This property may be less than the sum of 
    //              TDSCaps::MaxHwMixingStaticBuffers and 
    //              TDSCaps::MaxHwMixingStreamingBuffers. Resource tradeoffs 
    //              frequently occur. 
    // ----------------------------------------------------------------------
    __property dword MaxHwMixingAllBuffers = { 
    read=FGetMaxHwMixingAllBuffers, write=FSetMaxHwMixingAllBuffers, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    MaxHwMixingStaticBuffers
    // Description: The MaxHwMixingStaticBuffers property defines the total 
    //              number of mixing buffers the hardware offers for all 
    //              static buffers. 
    //              By comparing this property with 
    //              TDSCaps::FreeHwMixingStaticBuffers, a value for the 
    //              resources already allocated can be determined. 
    // ----------------------------------------------------------------------
    __property dword MaxHwMixingStaticBuffers = { 
    read=FGetMaxHwMixingStaticBuffers, write=FSetMaxHwMixingStaticBuffers, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    MaxHwMixingStreamingBuffers
    // Description: The MaxHwMixingStreamingBuffers property defines the 
    //              total number of mixing buffers the hardware offers for 
    //              all streaming buffers. 
    //              By comparing this property with 
    //              TDSCaps::FreeHwMixingStreamingBuffers, a value for the 
    //              resources already allocated can be determined. 
    // ----------------------------------------------------------------------
    __property dword MaxHwMixingStreamingBuffers = { 
    read=FGetMaxHwMixingStreamingBuffers, 
    write=FSetMaxHwMixingStreamingBuffers, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    MaxSecondarySampleRate
    // Description: The MaxSecondarySampleRate property defines the maximum 
    //              sample rate that the hardware's secondary sound buffers 
    //              can support. 
    // ----------------------------------------------------------------------
    __property dword MaxSecondarySampleRate = { 
    read=FGetMaxSecondarySampleRate, write=FSetMaxSecondarySampleRate, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    MinSecondarySampleRate
    // Description: The MinSecondarySampleRate property defines the minimum 
    //              sample rate that the hardware's secondary sound buffers 
    //              can support. 
    // ----------------------------------------------------------------------
    __property dword MinSecondarySampleRate = { 
    read=FGetMinSecondarySampleRate, write=FSetMinSecondarySampleRate, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    PlayCpuOverheadSwBuffers
    // Description: The PlayCpuOverheadSwBuffers property represents the 
    //              percentage of main processing cycles needed to mix sound 
    //              buffers located in system memory. 
    //              This value will depend on the bus type, processor type 
    //              and clock speed. 
    //              The overhead of a hardware buffer will be zero because 
    //              the mixing is done by the sound device. 
    // ----------------------------------------------------------------------
    __property dword PlayCpuOverheadSwBuffers = { 
    read=FGetPlayCpuOverheadSwBuffers, write=FSetPlayCpuOverheadSwBuffers, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    PrimaryBuffers
    // Description: The PrimaryBuffers property defines the number of primary 
    //              buffers that are supported. 
    //              This value will always be 1. 
    // ----------------------------------------------------------------------
    __property dword PrimaryBuffers = { read=FGetPrimaryBuffers, 
    write=FSetPrimaryBuffers, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property defines the size, in bytes, of the 
    //              internal structure used by this component. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, 
    default=sizeof(DSCAPS) }; 

    // ----------------------------------------------------------------------
    // Property:    TotalHwMemBytes
    // Description: The TotalHwMemBytes property defines the total amount, in 
    //              bytes, of the memory on the sound card that is used for 
    //              storing static sound buffers. 
    // ----------------------------------------------------------------------
    __property dword TotalHwMemBytes = { read=FGetTotalHwMemBytes, 
    write=FSetTotalHwMemBytes, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    UnlockTransferRateHwBuffers
    // Description: The UnlockTransferRateHwBuffers property describes the 
    //              rate, in kilobytes per second, at which data is 
    //              transferred to hardware static sound buffers when 
    //              TDx_SoundBuffer::Unlock() is called. 
    //              This value, along with the number of bytes, can be used 
    //              by applications to determine the time required for 
    //              TDx_SoundBuffer::Unlock() to execute. 
    //              The transfer rate for a software buffer is zero because 
    //              the data does not need to be tansferred anywhere. 
    // ----------------------------------------------------------------------
    __property dword UnlockTransferRateHwBuffers = { 
    read=FGetUnlockTransferRateHwBuffers, 
    write=FSetUnlockTransferRateHwBuffers, default=0 }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDSCaps component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDSCaps method or fget/fset. 
    //              eg. DS_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DS_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Flags
    // Description: The Flags property defines flags specifying the available 
    //              device capabilities. 
    //              The described effect applies when the flag is set. 
    // Flags:       DSCAPS_CERTIFIED -
    //                The sound card driver has been tested and certified by 
    //                Microsoft. 
    //              DSCAPS_CONTINUOUSRATE -
    //                Support for all sample rates falling between the values 
    //                of TDSCaps::MinSecondarySampleRate and 
    //                TDSCaps::MaxSecondarySampleRate is available. 
    //                This usually means actual output rates will be within 
    //                +/- 10 hertz (Hz) of the desired frequency. 
    //              DSCAPS_EMULDRIVER -
    //                There is no DirectSound driver installed for the 
    //                device, so it is being emulated through the Windows 
    //                waveform-audio functions. 
    //                Expect some degree of performance degradation. 
    //              DSCAPS_PRIMARY16BIT -
    //                Support for primary sound buffers with 16 bit samples 
    //                is available. 
    //              DSCAPS_PRIMARY8BIT -
    //                Support for primary sound buffers with 8 bit samples is 
    //                available. 
    //              DSCAPS_PRIMARYMONO -
    //                Support for monophonic primary sound buffers is 
    //                available. 
    //              DSCAPS_PRIMARYSTEREO -
    //                Support for stereo primary sound buffers is available. 
    //              DSCAPS_SECONDARY16BIT -
    //                Support for hardware mixed secondary sound buffers with 
    //                16 bit samples is available. 
    //              DSCAPS_SECONDARY8BIT -
    //                Support for hardware mixed secondary sound buffers with 
    //                8 bit samples is available. 
    //              DSCAPS_SECONDARYMONO -
    //                Support for hardware mixed monophonic secondary buffers 
    //                is available. 
    //              DSCAPS_SECONDARYSTEREO -
    //                Support for hardware mixed stereo secondary buffers is 
    //                available. 
    // ----------------------------------------------------------------------
    __property dword Flags = { read=FGetFlags, write=FSetFlags, default=0 }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDSCaps's internal DSCAPS structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Property:    Flags_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the Flags property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* Flags_Strings = { read=FGetFlags_Strings };

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDSCaps(TComponent* Owner); 
    virtual __fastcall ~TDSCaps(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DSCAPS* Internal_DSCAPS_Ptr = { read=FGetInternal_DSCAPS_Ptr, 
    nodefault }; 

    void __fastcall Internal_DSCAPS_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Object Inspector 'Flag' Access Methods
    // ----------------------------------------------------------------------
    TDSCaps_Flags_Set __fastcall FGetFlags_(); 
    void __fastcall FSetFlags_( TDSCaps_Flags_Set pFlags_ ); 

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetFlags(); 
    void __fastcall FSetFlags( dword pFlags ); 
    dword __fastcall FGetFreeHw3DAllBuffers(); 
    void __fastcall FSetFreeHw3DAllBuffers( dword pFreeHw3DAllBuffers ); 
    dword __fastcall FGetFreeHw3DStaticBuffers(); 
    void __fastcall FSetFreeHw3DStaticBuffers( dword pFreeHw3DStaticBuffers 
    ); 
    dword __fastcall FGetFreeHw3DStreamingBuffers(); 
    void __fastcall FSetFreeHw3DStreamingBuffers( dword 
    pFreeHw3DStreamingBuffers ); 
    dword __fastcall FGetFreeHwMemBytes(); 
    void __fastcall FSetFreeHwMemBytes( dword pFreeHwMemBytes ); 
    dword __fastcall FGetFreeHwMixingAllBuffers(); 
    void __fastcall FSetFreeHwMixingAllBuffers( dword pFreeHwMixingAllBuffers 
    ); 
    dword __fastcall FGetFreeHwMixingStaticBuffers(); 
    void __fastcall FSetFreeHwMixingStaticBuffers( dword 
    pFreeHwMixingStaticBuffers ); 
    dword __fastcall FGetFreeHwMixingStreamingBuffers(); 
    void __fastcall FSetFreeHwMixingStreamingBuffers( dword 
    pFreeHwMixingStreamingBuffers ); 
    dword __fastcall FGetMaxContigFreeHwMemBytes(); 
    void __fastcall FSetMaxContigFreeHwMemBytes( dword 
    pMaxContigFreeHwMemBytes ); 
    dword __fastcall FGetMaxHw3DAllBuffers(); 
    void __fastcall FSetMaxHw3DAllBuffers( dword pMaxHw3DAllBuffers ); 
    dword __fastcall FGetMaxHw3DStaticBuffers(); 
    void __fastcall FSetMaxHw3DStaticBuffers( dword pMaxHw3DStaticBuffers ); 
    dword __fastcall FGetMaxHw3DStreamingBuffers(); 
    void __fastcall FSetMaxHw3DStreamingBuffers( dword 
    pMaxHw3DStreamingBuffers ); 
    dword __fastcall FGetMaxHwMixingAllBuffers(); 
    void __fastcall FSetMaxHwMixingAllBuffers( dword pMaxHwMixingAllBuffers 
    ); 
    dword __fastcall FGetMaxHwMixingStaticBuffers(); 
    void __fastcall FSetMaxHwMixingStaticBuffers( dword 
    pMaxHwMixingStaticBuffers ); 
    dword __fastcall FGetMaxHwMixingStreamingBuffers(); 
    void __fastcall FSetMaxHwMixingStreamingBuffers( dword 
    pMaxHwMixingStreamingBuffers ); 
    dword __fastcall FGetMaxSecondarySampleRate(); 
    void __fastcall FSetMaxSecondarySampleRate( dword pMaxSecondarySampleRate 
    ); 
    dword __fastcall FGetMinSecondarySampleRate(); 
    void __fastcall FSetMinSecondarySampleRate( dword pMinSecondarySampleRate 
    ); 
    dword __fastcall FGetPlayCpuOverheadSwBuffers(); 
    void __fastcall FSetPlayCpuOverheadSwBuffers( dword 
    pPlayCpuOverheadSwBuffers ); 
    dword __fastcall FGetPrimaryBuffers(); 
    void __fastcall FSetPrimaryBuffers( dword pPrimaryBuffers ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    dword __fastcall FGetTotalHwMemBytes(); 
    void __fastcall FSetTotalHwMemBytes( dword pTotalHwMemBytes ); 
    dword __fastcall FGetUnlockTransferRateHwBuffers(); 
    void __fastcall FSetUnlockTransferRateHwBuffers( dword 
    pUnlockTransferRateHwBuffers ); 

    // ----------------------------------------------------------------------
    // Flag_Strings Access Methods
    // ----------------------------------------------------------------------
    TStringList* __fastcall FGetFlags_Strings();

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DSCAPS* __fastcall FGetInternal_DSCAPS_Ptr(); 
    void __fastcall FSetInternal_DSCAPS( DSCAPS* pDSCAPS ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Flags_Strings Variables
    // ----------------------------------------------------------------------
    TStringList* fFlags_Strings;

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DSCAPS fDSCAPS; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
