#ifndef TDx_SoundCaptureH
#define TDx_SoundCaptureH

// ==========================================================================
// File:         TDx_SoundCapture.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons),
//               
// Description:  This file defines the TDx_SoundCapture Component 
//
// "TDx_Sound_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Sound_Library_Defns.H"
#include "TDx_Sound_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by TDx_SoundCapture methods.
class TDSCBufferDesc;
class TDx_SoundCaptureBuffer;
class TDSCCaps;

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// Definition for the OnDSCaptureEnumerate callback event.
typedef	__fastcall void	(__closure*     TDx_DSCaptureEnumerateCallback) 
                        ( TObject*      Sender, 
                          GUID*         Guid, 
                          AnsiString    DriverDescription, 
                          AnsiString    DriverModule, 
                          void*         Context, 
                          bool&         Finished ); 

// ==========================================================================
// Class:       TDx_SoundCapture
// Description: The TDx_SoundCapture component wraps the DirectX 
//              IDirectSoundCapture interface which is used in the creation 
//              of sound capture buffers. 
//              
//              This component is used to create capture buffers and retrieve 
//              the capture capabilities of the sound hardware. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDx_SoundCapture : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDx_SoundCapture : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnCreate()
    // Description: The OnCreate() event is triggered by the 
    //              TDx_SoundCapture::Create() method after it has 
    //              successfully created the internal LPDIRECTSOUNDCAPTURE 
    //              used within the TDx_SoundCapture component. 
    //              
    //              Note that since this event is only called upon successful 
    //              creation, this event is an ideal place for code that is 
    //              to be activated only once, immediately upon the 
    //              successful creation of the LPDIRECTSOUNDCAPTURE 
    //              interface. 
    //              
    //              For example, you could retrieve the TDx_SoundCapture 
    //              component's capabilities, Create() dependant components, 
    //              allocate memory, etc. 
    //              
    //              When used in conjunction with the 
    //              TDx_SoundCapture::OnDestroy() event, you can fully 
    //              automate and insulate your code on a per-component basis. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnCreate = {read=FOnCreate, write=FOnCreate, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnDestroy()
    // Description: The OnDestroy() event is triggered by the 
    //              TDx_SoundCapture::Destroy() method, just before the 
    //              TDx_SoundCapture internal LPDIRECTSOUNDCAPTURE interface 
    //              is destroyed. 
    //              
    //              Note: This event is an ideal place to place code that 
    //              reverses the results of the TDx_SoundCapture::OnCreate() 
    //              event. For example, assume you have already setup the 
    //              TDx_SoundCapture::OnCreate() event to retrieve the 
    //              component's capabilities, Create() a dependant component 
    //              and allocate some memory; 
    //              You can then fully automate and insulate your code on a 
    //              per-component basis by placing code in the 
    //              TDx_SoundCapture::OnDestroy() event to deallocate memory, 
    //              Destroy() the dependant components and, when required, 
    //              destroy the capabilities component. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnDestroy = {read=FOnDestroy, write=FOnDestroy, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDx_SoundCapture component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Callback:    OnDSCaptureEnumerate()
    // Description: The OnDSCaptureEnumerate event is used to process 
    //              information returned by 
    //              TDx_SoundCapture::DSCaptureEnumerate(). 
    //              
    //              Set the Finished parameter to halt the enumeration 
    //              process. 
    // ----------------------------------------------------------------------
    __property TDx_DSCaptureEnumerateCallback OnDSCaptureEnumerate = 
    {read=FOnDSCaptureEnumerate, write=FOnDSCaptureEnumerate, nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Created
    // Description: The Created property is true if the internal 
    //              LPDIRECTSOUNDCAPTURE used in this component has been 
    //              successfully created, otherwise Created is false. 
    //              
    //              To create the internal LPDIRECTSOUNDCAPTURE, call the 
    //              TDx_SoundCapture::Create() method. 
    // ----------------------------------------------------------------------
    __property bool Created = { read=FGetCreated, write=FSetCreated, 
    default=false }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDx_SoundCapture method or 
    //              fget/fset. eg. DS_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DS_OK }; 

    // ----------------------------------------------------------------------
    // Method:      Create()
    // Description: The Create() method is used to automatically create the 
    //              internal LPDIRECTSOUNDCAPTURE interface used in the 
    //              TDx_SoundCapture component and must be called before any 
    //              methods of the TDx_SoundCapture component will function. 
    // Params:      pGUID - 
    //                The GUID parameter identifies the sound capture device to 
    //                use when creating the sound interface. 
    //                The TDx_SoundCapture::DSCaptureEnumerate method can be 
    //                used to obtain the GUID's of the available sound capture 
    //                devices. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Create( GUID* pGUID ); 

    // ----------------------------------------------------------------------
    // Method:      CreateCaptureBuffer()
    // Description: The TDx_SoundCapture::CreateCaptureBuffer method will 
    //              create a capture buffer. 
    //              TDx_SoundCapture cannot extract multiple sounds from one 
    //              input sound and only one capture buffer may exist per 
    //              device. 
    //              
    //              You can use TDx_SoundCaptureBuffer::Create() method to 
    //              achieve the same result as this method with the added 
    //              advantage of being able to use the 
    //              TDx_SoundCaptureBuffer::OnCreate() event. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DSERR_INVALIDPARAM 
    //              DSERR_BADFORMAT 
    //              DSERR_GENERIC 
    //              DSERR_NODRIVER 
    //              DSERR_OUTOFMEMORY 
    //              DSERR_UNINITIALIZED 
    // Params:      pCBufferDesc - 
    //                The CBufferDesc parameter references a TDSCBufferDesc 
    //                component holding the properties of the capture buffer to 
    //                be created. 
    //              pCaptureBuffer - 
    //                The CaptureBuffer parameter will reference the new 
    //                capture buffer if this method returns successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall CreateCaptureBuffer( TDSCBufferDesc* 
    pCBufferDesc, TDx_SoundCaptureBuffer* pCaptureBuffer ); 

    // ----------------------------------------------------------------------
    // Method:      DSCaptureEnumerate()
    // Description: The TDx_Sound::DSCaptureEnumerate method will scan all 
    //              the sound capture devices installed on the system and 
    //              call the TDx_SoundCapture::OnDSCaptureEnumCallback() 
    //              event for each device. 
    //              
    //              There is no need to write a manual callback function, 
    //              just place your code in the 
    //              TDx_SoundCapture::OnDSCaptureEnumCallback() event. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DSERR_INVALIDPARAM 
    // Params:      pContext - 
    //                The Context parameter references an application defined 
    //                object that is passed to the callback event for each 
    //                device enumerated. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall DSCaptureEnumerate( LPVOID pContext ); 

    // ----------------------------------------------------------------------
    // Method:      Destroy()
    // Description: The Destroy() method is used to automatically destroy the 
    //              internal LPDIRECTSOUNDCAPTURE interface used in the 
    //              TDx_SoundCapture component and should be called when the 
    //              internal interface is no longer required. 
    //              
    //              Note: This method is called by the component's 
    //              destructor. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Destroy(); 

    // ----------------------------------------------------------------------
    // Method:      GetCaps()
    // Description: The TDx_SoundCapture::GetCaps method will return the 
    //              capture device capabilities. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DSERR_INVALIDPARAM 
    //              DSERR_UNSUPPORTED 
    //              DSERR_NODRIVER 
    //              DSERR_OUTOFMEMORY 
    //              DSERR_UNINITIALIZED 
    // Params:      pCCaps - 
    //                The CCaps parameter references a TDSCCaps component for 
    //                holding the capabilities of the capture device if this 
    //                method returns successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetCaps( TDSCCaps* pCCaps ); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDx_SoundCapture(TComponent* Owner); 
    virtual __fastcall ~TDx_SoundCapture(); 

    // ----------------------------------------------------------------------
    // The following properties and methods are used internally by 
    // TDx_Sound_Library and should not be used. 
    // ----------------------------------------------------------------------
    __property LPDIRECTSOUNDCAPTURE Internal_LPDIRECTSOUNDCAPTURE = { 
    read=FGetInternal_LPDIRECTSOUNDCAPTURE, 
    write=FSetInternal_LPDIRECTSOUNDCAPTURE, nodefault }; 
    __property LPDIRECTSOUNDCAPTURE* Internal_LPDIRECTSOUNDCAPTURE_Ptr = { 
    read=FGetInternal_LPDIRECTSOUNDCAPTURE_Ptr, nodefault }; 
    void __fastcall Internal_LPDIRECTSOUNDCAPTURE_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Method:   Internal_DSCaptureEnumerateCallback()
    // Description: The Internal_DSCaptureEnumerateCallback method is used 
    //              internally by the TDx_SoundCapture component as a 
    //              callback function. 
    //              It translates DirectX-style parameters to BCB-style 
    //              parameters before calling the 
    //              TDx_SoundCapture::OnDSCaptureEnumCallback() event. 
    // Params:      pGuid - 
    //                This parameter is used internally. 
    //              pDescription - 
    //                This parameter is used internally. 
    //              pModule - 
    //                This parameter is used internally. 
    //              pContext - 
    //                This parameter is used internally. 
    // ----------------------------------------------------------------------
    static BOOL __stdcall Internal_DSCaptureEnumerateCallback( LPGUID pGuid, 
    LPCSTR pDescription, LPCSTR pModule, LPVOID pContext ); 

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    bool __fastcall FGetCreated(); 
    void __fastcall FSetCreated( bool pCreated ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Interface Access
    // ----------------------------------------------------------------------
    LPDIRECTSOUNDCAPTURE __fastcall FGetInternal_LPDIRECTSOUNDCAPTURE(); 
    void __fastcall FSetInternal_LPDIRECTSOUNDCAPTURE( LPDIRECTSOUNDCAPTURE 
    pLPDIRECTSOUNDCAPTURE ); 
    LPDIRECTSOUNDCAPTURE* __fastcall FGetInternal_LPDIRECTSOUNDCAPTURE_Ptr(); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    bool fCreated; 
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Interface Variables 
    // ----------------------------------------------------------------------
    LPDIRECTSOUNDCAPTURE fLPDIRECTSOUNDCAPTURE; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Event FOnCreate;
    TDx_Event FOnDestroy;
    TDx_Error FOnError;

    // ----------------------------------------------------------------------
    // Callback Variables
    // ----------------------------------------------------------------------
    TDx_DSCaptureEnumerateCallback FOnDSCaptureEnumerate;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
