#ifndef TDx_Sound_Library_DefnsH
#define TDx_Sound_Library_DefnsH

// ==========================================================================
// File:        TDx_Sound_Library_Defns.H
// Description: This file contains standard definitions used with the
//              TDx_Sound_Library components.
//
// "TDx_Sound_Library"
// (c) 2002, BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

#include "mmsystem.h"
#include "dsound.h"
#include "io.h"
#include "fcntl.h"
#include "sys\stat.h"
#include "TDx_Library_Defns.H"

// --------------------------------------------------------------------------
// Note: There seems to be a missing/incorrect flag in the SDK.
//       This #ifdef fixes the issues that arise when compiling.
// --------------------------------------------------------------------------
#ifndef DSCCAPS_CERTIFIED
#define DSCCAPS_CERTIFIED    DSCAPS_CERTIFIED
#endif
//---------------------------------------------------------------------------
// Structure: TDX_WAVEFORMAT
// Description: Used instead of a WAVEFORMATEX
//---------------------------------------------------------------------------
typedef struct
    {
    word        format;                             // = 0x01 == WAVE_FORMAT_PCM
    word        num_channels;                       // = 0x01 (mono), 0x02 (stereo)
    dword       sample_rate;                        // binary, in Hertz (Hz)
    dword       bytes_per_second;                   // calculated
    word        bytes_per_sample;                   // 1 = 8 bit, 2 = 8 bit stereo or 16 bit mono, 4 = 16 bit stereo
    word        bits_per_sample;                    // calculated
    } TDX_WAVE_FORMAT;

typedef struct // 12 bytes total
    {
    byte        id[4];                              // = ( 'R', 'I', 'F', 'F' );
    dword       size;                               // length of entire file - 8 bytes
    byte        data[4];                            // = { 'W', 'A', 'V', 'E' };
    } TDX_RIFF_CHUNK;

typedef struct // 24 bytes total
    {
    byte        id[4];                              // = { 'f', 'm', 't', ' ' };
    dword       size;                               // length_of_format_chunk == 0x10;
    TDX_WAVE_FORMAT format;                         // like a WAVEFORMATEX without cbSize
    } TDX_FORMAT_CHUNK;

typedef struct // 8 + undefined raw sound data size (bytes)
    {
    byte        id[4];                              // = { 'd', 'a', 't', 'a' };
    dword       size;                               // size of raw data in bytes. default = 0
//  byte[]      raw_byte_data;                      // raw sound data goes here normally
    } TDX_DATA_CHUNK;

//---------------------------------------------------------------------------

typedef struct       // used for loading / saving .wav files
    {
    TDX_RIFF_CHUNK      riff_chunk;
    TDX_FORMAT_CHUNK    format_chunk;
    TDX_DATA_CHUNK      data_chunk;
    } TDX_WAVE_FILE_FORMAT;

extern TDX_WAVE_FILE_FORMAT DEFAULT_TDX_WAVE_FILE_FORMAT;

//---------------------------------------------------------------------------

#endif
